@@ -1,5 +1,6 @@
 import abc
 import logging
+from contextlib import suppress
 from fnmatch import fnmatch
 from operator import attrgetter
 
@@ -10,6 +11,7 @@ from ..errors import (
     ErrorDeleteDistinguishedFolder,
     ErrorFolderNotFound,
     ErrorItemNotFound,
+    ErrorRecoverableItemsAccessDenied,
     InvalidTypeError,
 )
 from ..fields import (
@@ -47,6 +49,13 @@ from .queryset import SingleFolderQuerySet
 
 log = logging.getLogger(__name__)
 
+DELETE_FOLDER_ERRORS = (
+    ErrorAccessDenied,
+    ErrorCannotDeleteObject,
+    ErrorCannotEmptyFolder,
+    ErrorItemNotFound,
+)
+
 
 class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
     """Base class for all classes that implement a folder."""
@@ -229,27 +238,41 @@ class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
         :return:
         """
         from .known_folders import (
+            ApplicationData,
             Calendar,
             Contacts,
             ConversationSettings,
+            CrawlerData,
+            DlpPolicyEvaluation,
+            FreeBusyCache,
             GALContacts,
             Messages,
             RecipientCache,
+            RecoveryPoints,
             Reminders,
             RSSFeeds,
+            Signal,
+            SwssItems,
             Tasks,
         )
 
         for folder_cls in (
-            Messages,
-            Tasks,
+            ApplicationData,
             Calendar,
-            ConversationSettings,
             Contacts,
+            ConversationSettings,
+            CrawlerData,
+            DlpPolicyEvaluation,
+            FreeBusyCache,
             GALContacts,
-            Reminders,
-            RecipientCache,
+            Messages,
             RSSFeeds,
+            RecipientCache,
+            RecoveryPoints,
+            Reminders,
+            Signal,
+            SwssItems,
+            Tasks,
         ):
             if folder_cls.CONTAINER_CLASS == container_class:
                 return folder_cls
@@ -307,10 +330,8 @@ class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
     @classmethod
     def get_item_field_by_fieldname(cls, fieldname):
         for item_model in cls.supported_item_models:
-            try:
+            with suppress(InvalidField):
                 return item_model.get_field_by_fieldname(fieldname)
-            except InvalidField:
-                pass
         raise InvalidField(f"{fieldname!r} is not a valid field name on {cls.supported_item_models}")
 
     def get(self, *args, **kwargs):
@@ -402,25 +423,37 @@ class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
     def wipe(self, page_size=None, chunk_size=None, _seen=None, _level=0):
         # Recursively deletes all items in this folder, and all subfolders and their content. Attempts to protect
         # distinguished folders from being deleted. Use with caution!
+        from .known_folders import Audits
+
         _seen = _seen or set()
         if self.id in _seen:
             raise RecursionError(f"We already tried to wipe {self}")
         if _level > 16:
             raise RecursionError(f"Max recursion level reached: {_level}")
         _seen.add(self.id)
+        if isinstance(self, Audits):
+            # Shortcircuit because this folder can have many items that are all non-deletable
+            log.warning("Cannot wipe audits folder %s", self)
+            return
+        if self.is_distinguished and "recoverableitems" in self.DISTINGUISHED_FOLDER_ID:
+            log.warning("Cannot wipe recoverable items folder %s", self)
+            return
         log.warning("Wiping %s", self)
         has_distinguished_subfolders = any(f.is_distinguished for f in self.children)
         try:
             if has_distinguished_subfolders:
-                self.empty(delete_sub_folders=False)
+                self.empty()
             else:
                 self.empty(delete_sub_folders=True)
-        except (ErrorAccessDenied, ErrorCannotEmptyFolder, ErrorItemNotFound):
+        except ErrorRecoverableItemsAccessDenied:
+            log.warning("Access denied to %s. Skipping", self)
+            return
+        except DELETE_FOLDER_ERRORS:
             try:
                 if has_distinguished_subfolders:
                     raise  # We already tried this
-                self.empty(delete_sub_folders=False)
-            except (ErrorAccessDenied, ErrorCannotEmptyFolder, ErrorItemNotFound):
+                self.empty()
+            except DELETE_FOLDER_ERRORS:
                 log.warning("Not allowed to empty %s. Trying to delete items instead", self)
                 kwargs = {}
                 if page_size is not None:
@@ -429,7 +462,7 @@ class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
                     kwargs["chunk_size"] = chunk_size
                 try:
                     self.all().delete(**kwargs)
-                except (ErrorAccessDenied, ErrorCannotDeleteObject, ErrorItemNotFound):
+                except DELETE_FOLDER_ERRORS:
                     log.warning("Not allowed to delete items in %s", self)
         _level += 1
         for f in self.children:
@@ -880,20 +913,16 @@ class Folder(BaseFolder):
             # The returned XML may contain neither folder class nor name. In that case, we default to the generic
             # Folder class.
             if folder.name:
-                try:
+                with suppress(KeyError):
                     # TODO: fld_class.LOCALIZED_NAMES is most definitely neither complete nor authoritative
                     folder_cls = root.folder_cls_from_folder_name(folder_name=folder.name, locale=root.account.locale)
                     log.debug("Folder class %s matches localized folder name %s", folder_cls, folder.name)
-                except KeyError:
-                    pass
             if folder.folder_class and folder_cls == Folder:
-                try:
+                with suppress(KeyError):
                     folder_cls = cls.folder_cls_from_container_class(container_class=folder.folder_class)
                     log.debug(
                         "Folder class %s matches container class %s (%s)", folder_cls, folder.folder_class, folder.name
                     )
-                except KeyError:
-                    pass
             if folder_cls == Folder:
                 log.debug("Fallback to class Folder (folder_class %s, name %s)", folder.folder_class, folder.name)
         return folder_cls(root=root, **{f.name: getattr(folder, f.name) for f in folder.FIELDS})
